@@ -59,7 +59,7 @@ def test_sqa_appbuild_results_record_model_creation(sqa_app_build_result):
 def test_sqa_test_results_record_schema_validation(sqa_test_result):
     model = SqaTestResult(sqa_test_result.dict())
     model.validate_schema()
-    model.testResult = "failed"
+    model.testResult = "fail"
     model.validate_schema()
 
     model.testCaseId = 4
