@@ -1,14 +1,21 @@
+import json
 import os
 
 from utf_queue_client.clients.ubai_artifact_upload_request_producer import (
     UbaiArtifactUploadRequestProducer,
 )
+from ubai_client.apis import ArtifactApi
 from urllib import parse
 import click
 from typing import Iterable, Tuple
 from time import sleep
-from utf_queue_client.scripts import setup_telemetry
+from utf_queue_client.scripts import setup_telemetry, get_or_make_ca_file
 from retry import retry
+import requests
+from otel_extensions import instrumented
+from opentelemetry import trace
+
+MULTIPART_SIZE_THRESHOLD = 1000000
 
 
 @click.command()
@@ -89,11 +96,13 @@ def cli(
             sleep(2)
 
 
+@instrumented
 def upload_artifact_through_queue(url, client_id, file_path, metadata_dict):
     client = UbaiArtifactUploadRequestProducer(url, client_id)
     client.upload_artifact(file_path, metadata=metadata_dict)
 
 
+@instrumented
 def upload_artifact_direct(file_path, metadata_dict):
     (
         name,
@@ -101,9 +110,31 @@ def upload_artifact_direct(file_path, metadata_dict):
         contents,
         base64_content,
     ) = UbaiArtifactUploadRequestProducer.extract_payload(file_path)
-    UbaiArtifactUploadRequestProducer.upload_artifact_direct(
-        name, extension, base64_content, metadata_dict
-    )
+    span = trace.get_current_span()
+    span.set_attribute("artifact.name", name)
+    span.set_attribute("artifact.extension", extension)
+    span.set_attribute("artifact.length", len(contents))
+    for key in metadata_dict:
+        span.set_attribute(f"metadata.{key}", metadata_dict[key])
+    if len(contents) > MULTIPART_SIZE_THRESHOLD:
+        span.set_attribute("multipart_upload", True)
+        # use multipart transfer endpoint
+        artifact_api = ArtifactApi()
+        with open(file_path, "rb") as f:
+            files = {"file": (name + extension, f)}
+            resp = requests.post(
+                f"{artifact_api.api_client.configuration.host}/api/v1/artifact-upload-file",
+                files=files,
+                data={"metadata": json.dumps(metadata_dict)},
+                verify=get_or_make_ca_file()
+            )
+            if resp.status_code != 200 and resp.status_code != 201:
+                raise RuntimeError(resp.content)
+    else:
+        span.set_attribute("multipart_upload", False)
+        UbaiArtifactUploadRequestProducer.upload_artifact_direct(
+            name, extension, base64_content, metadata_dict
+        )
 
 
 if __name__ == "__main__":
